// Ansteuern eines Standard-LCD mit 4-Bit-Schnittstelle
// und Abfrage eines Joysticks
// Pin 1.0: RS
// Pin 1.1: E
// Pins 1.4 bis 1.7: D4 bis D7
// Pins 1.2 und 1.3: Eingnge fr Joystick x/y (Ausgang Spannungsteiler GND...VCC)
// Version: 2012-11-07

#include "display_joystick.h"
#include "io430.h"
#include <stdbool.h>

static const unsigned char RS = BIT0;
static const unsigned char E = BIT1;
static const unsigned char A = BIT2;
static const unsigned int CH_A = INCH_2;
static const unsigned char B = BIT3;
static const unsigned int CH_B = INCH_3;

// Daten in den oberen vier Bits
static void sendNibble(unsigned char data)
{
  P1OUT = A | B | data; // Pull-up fr Encoder
  P1OUT |= E;
  waitOneMillisecond();
  P1OUT &= ~E;  
  waitOneMillisecond(); // schneller wre, auf die Rckmeldung des Displays zu warten,
                        // aber so sparen wir eine Leitung
}

static void sendByte(unsigned char data)
{
  sendNibble(data & 0xF0u);
  sendNibble(data << 4);
}

static void sendByteRAM(unsigned char data)
{
  sendNibble(data & 0xF0u | RS);
  sendNibble((data << 4) | RS);
}

void initialize(void)
{
  WDTCTL = WDTPW | WDTHOLD;
  
  BCSCTL1 = CALBC1_1MHZ;
  DCOCTL = CALDCO_1MHZ;
  
  ADC10CTL0 = ADC10SHT_2 | ADC10ON;
  
  TACTL = MC_2 | TASSEL_2 | TACLR;
  
  P1DIR = ~(A | B);
  
  waitMilliseconds(50);
  sendNibble(0x30u); // Initialisierungsequenz fr den Warmstart
  waitMilliseconds(50);
  sendNibble(0x30u);
  waitMilliseconds(50);
  sendNibble(0x30u);
  sendNibble(0x20u);
  sendByte(0x20u); // Ansteuerung nur einer Zeile, weil fr unser LCD sonst negative Kontrastspannung ntig
  sendByte(0x01u); // lschen  
  sendByte(0x0Cu); // Display an, Cursor aus, Blinken aus
  sendByte(0x06u); // den Inhalt nicht mit dem Cursor schieben
}

int readAnalog(int channel)
{
  ADC10CTL0 &= ~ENC;
  
  if(channel == 0)
  {
    ADC10CTL1 = CH_A;
    ADC10AE0 = A;
  }
  else
  {
    ADC10CTL1 = CH_B;
    ADC10AE0 = B;
  }

  ADC10CTL0 |= ENC;
  ADC10CTL0 |= ADC10SC;
  while ((ADC10CTL1 & ADC10BUSY) != 0)
  {}
  return ADC10MEM;
}

void clear(void)
{
  sendByte(0x01u);
}

void putCharacter(unsigned char row, unsigned char column, char character)
{
  sendByte(0x02u); // "home"
  sendByte(0x80u + 0x40u*row + column); // Adresse
  sendByteRAM((unsigned char)character);
}

void writeString(char s[])
{
  bool done = false;
  for(int i = 0; i < 8; i++)
  {
    if(s[i] == 0)
    {
      done = true;
    }
    if(!done)
    {
      putCharacter(0, i, s[i]);
    }
    else
    {
      putCharacter(0, i, ' ');      
    }
  }
}

void writeNumber(unsigned int a)
{
  if(a == 0)
  {
    writeString("       0");
  }
  else
  {
    for(int i = 0; i < 8; i++) // Vorsicht: Stellen von hinten gezhlt
    {
      if(a > 0)
      {
        putCharacter(0, 7-i, '0' + a % 10);
        a /= 10;
      }
      else
      {
        putCharacter(0, 7-i, ' ');
      }
    }
  }
}

void showCursor(bool visible, bool blinking)
{
  sendByte(0x0Cu | (visible?2u:0u) | (blinking?1u:0u));
}

void shiftCursor(bool right)
{
  sendByte(0x10u | (right?0u:1u));
}

void moveCursor(unsigned char row, unsigned char column)
{
  sendByte(0x02u); // "return home"  
  sendByte(0x80u + column + 0x40u*row);
}

void waitOneMillisecond(void)
{
  unsigned short tStart = TAR;
  unsigned short tEnd = tStart + 1000;
  // ggf. Zeit zum Arbeiten am Anfang und Ende schaffen
  if(tEnd > 0xFFF0)
  {
    tEnd = 0xFFF0;
  }
  else if(tEnd < 0xF)
  {
    tEnd = 0xF;
  }
  
  if(tEnd < tStart) // Hatten wir einen berlauf beim Berechnen von tEnd?
  {
    while(TAR > tEnd)
    {   
    }
  }
  while(TAR < tEnd)
  {
  }
}

void waitMilliseconds(unsigned int duration)
{
  for(unsigned int i = 0; i < duration; i++)
  {
    waitOneMillisecond();
  }
}